@@ -69,7 +69,9 @@ def _attach_interfaces_to_driver(bare_driver, node, hw_type):
              the requested implementation is not compatible with it.
     """
     for iface in _INTERFACE_LOADERS:
-        impl_name = getattr(node, '%s_interface' % iface)
+        iface_name = '%s_interface' % iface
+        impl_name = node.instance_info.get(iface_name,
+                                           getattr(node, iface_name))
         impl = get_interface(hw_type, iface, impl_name)
         setattr(bare_driver, iface, impl)
 
@@ -130,20 +132,24 @@ def default_interface(hw_type, interface_type,
     :raises: InterfaceNotFoundInEntrypoint if the entry point was not found.
     :raises: NoValidDefaultForInterface if no default interface can be found.
     """
+
     factory = _INTERFACE_LOADERS[interface_type]
-    # Explicit interface defaults
-    additional_defaults = {
-        'storage': 'noop'
-    }
 
     # The fallback default from the configuration
     impl_name = getattr(CONF, 'default_%s_interface' % interface_type)
-    if impl_name is None:
-        impl_name = additional_defaults.get(interface_type)
 
     if impl_name is not None:
-        # Check that the default is correct for this type
-        get_interface(hw_type, interface_type, impl_name)
+        try:
+            # Check that the default is correct for this type
+            get_interface(hw_type, interface_type, impl_name)
+        except exception.IncompatibleInterface:
+            node_info = ""
+            if node is not None:
+                node_info = _(' node %s with') % node
+            raise exception.NoValidDefaultForInterface(
+                interface_type=interface_type, driver=driver_name,
+                node_info=node_info)
+
     else:
         supported = getattr(hw_type,
                             'supported_%s_interfaces' % interface_type)
@@ -204,20 +210,29 @@ def check_and_update_node_interfaces(node, hw_type=None):
         # NOTE(dtantsur): objects raise NotImplementedError on accessing fields
         # that are known, but missing from an object. Thus, we cannot just use
         # getattr(node, field_name, None) here.
+        set_default = True
+        if 'instance_info' in node and field_name in node.instance_info:
+            impl_name = node.instance_info.get(field_name)
+            if impl_name is not None:
+                # Check that the provided value is correct for this type
+                get_interface(hw_type, iface, impl_name)
+                set_default = False
+
         if field_name in node:
             impl_name = getattr(node, field_name)
             if impl_name is not None:
                 # Check that the provided value is correct for this type
                 get_interface(hw_type, iface, impl_name)
-                # Not changing the result, proceeding with the next interface
-                continue
+                set_default = False
 
-        impl_name = default_interface(hw_type, iface,
-                                      driver_name=node.driver, node=node.uuid)
+        if set_default:
+            impl_name = default_interface(hw_type, iface,
+                                          driver_name=node.driver,
+                                          node=node.uuid)
 
-        # Set the calculated default and set result to True
-        setattr(node, field_name, impl_name)
-        result = True
+            # Set the calculated default and set result to True
+            setattr(node, field_name, impl_name)
+            result = True
 
     return result
 
@@ -300,9 +315,9 @@ class BaseDriverFactory(object):
     This is subclassed to load both main drivers and extra interfaces.
     """
 
-    # NOTE(deva): loading the _extension_manager as a class member will break
-    #             stevedore when it loads a driver, because the driver will
-    #             import this file (and thus instantiate another factory).
+    # NOTE(tenbrae): loading the _extension_manager as a class member will
+    #             break stevedore when it loads a driver, because the driver
+    #             will import this file (and thus instantiate another factory).
     #             Instead, we instantiate a NameDispatchExtensionManager only
     #             once, the first time DriverFactory.__init__ is called.
     _extension_manager = None
@@ -328,12 +343,12 @@ class BaseDriverFactory(object):
     def get_driver(self, name):
         return self[name].obj
 
-    # NOTE(deva): Use lockutils to avoid a potential race in eventlet
+    # NOTE(tenbrae): Use lockutils to avoid a potential race in eventlet
     #             that might try to create two driver factories.
     @classmethod
     @lockutils.synchronized(EM_SEMAPHORE)
     def _init_extension_manager(cls):
-        # NOTE(deva): In case multiple greenthreads queue up on this lock
+        # NOTE(tenbrae): In case multiple greenthreads queue up on this lock
         #             before _extension_manager is initialized, prevent
         #             creation of multiple NameDispatchExtensionManagers.
         if cls._extension_manager:
@@ -362,8 +377,8 @@ class BaseDriverFactory(object):
                         'configuration file.',
                         ', '.join(duplicated_drivers))
 
-        # NOTE(deva): Drivers raise "DriverLoadError" if they are unable to be
-        #             loaded, eg. due to missing external dependencies.
+        # NOTE(tenbrae): Drivers raise "DriverLoadError" if they are unable to
+        #             be loaded, eg. due to missing external dependencies.
         #             We capture that exception, and, only if it is for an
         #             enabled driver, raise it from here. If enabled driver
         #             raises other exception type, it is wrapped in
@@ -371,7 +386,7 @@ class BaseDriverFactory(object):
         #             caused it, and raised. If the exception is for a
         #             non-enabled driver, we suppress it.
         def _catch_driver_not_found(mgr, ep, exc):
-            # NOTE(deva): stevedore loads plugins *before* evaluating
+            # NOTE(tenbrae): stevedore loads plugins *before* evaluating
             #             _check_func, so we need to check here, too.
             if ep.name in cls._enabled_driver_list:
                 if not isinstance(exc, exception.DriverLoadError):
