@@ -77,6 +77,10 @@ class PortAlreadyExists(Conflict):
     _msg_fmt = _("A port with UUID %(uuid)s already exists.")
 
 
+class PortDuplicateName(Conflict):
+    _msg_fmt = _("A port with name %(name)s already exists.")
+
+
 class PortgroupAlreadyExists(Conflict):
     _msg_fmt = _("A portgroup with UUID %(uuid)s already exists.")
 
@@ -241,7 +245,8 @@ class NoValidDefaultForInterface(InvalidParameterValue):
     #             because node_info could be an empty string. If node_info
     #             is not empty, it should start with a space.
     _msg_fmt = _("For%(node_info)s hardware type '%(driver)s', no default "
-                 "value found for %(interface_type)s interface.")
+                 "value found for %(interface_type)s interface could be "
+                 "determined. Please ensure the interfaces are enabled.")
 
 
 class ImageNotFound(NotFound):
@@ -331,10 +336,9 @@ class ConductorAlreadyRegistered(IronicException):
 
 
 class ConductorHardwareInterfacesAlreadyRegistered(IronicException):
-    _msg_fmt = _("At least one of these (hardware type %(hardware_type)s, "
-                 "interface type %(interface_type)s, interfaces "
-                 "%(interfaces)s) combinations are already registered for "
-                 "this conductor.")
+    _msg_fmt = _("Duplication detected for hardware_type, interface_type "
+                 "and interface combinations for this conductor while "
+                 "registering the row %(row)s")
 
 
 class PowerStateFailure(InvalidState):
@@ -383,7 +387,7 @@ class ImageNotAuthorized(NotAuthorized):
     _msg_fmt = _("Not authorized for image %(image_id)s.")
 
 
-class InvalidImageRef(Invalid):
+class InvalidImageRef(InvalidParameterValue):
     _msg_fmt = _("Invalid image href %(image_href)s.")
 
 
@@ -708,6 +712,14 @@ class InvalidDeployTemplate(Invalid):
     _msg_fmt = _("Deploy template invalid: %(err)s.")
 
 
+class InvalidKickstartTemplate(Invalid):
+    _msg_fmt = _("The kickstart template is missing required variables")
+
+
+class InvalidKickstartFile(Invalid):
+    _msg_fmt = _("The kickstart file is not valid.")
+
+
 class IBMCError(DriverOperationError):
     _msg_fmt = _("IBMC exception occurred on node %(node)s. Error: %(error)s")
 
@@ -774,19 +786,6 @@ class UnknownArgument(ClientSideError):
         }
 
 
-class MissingArgument(ClientSideError):
-    def __init__(self, argname, msg=''):
-        self.argname = argname
-        super(MissingArgument, self).__init__(msg)
-
-    @property
-    def faultstring(self):
-        return _('Missing argument: "%(argname)s"%(msg)s') % {
-            'argname': self.argname,
-            'msg': self.msg and ": " + self.msg or ""
-        }
-
-
 class UnknownAttribute(ClientSideError):
     def __init__(self, fieldname, attributes, msg=''):
         self.fieldname = fieldname
@@ -813,3 +812,13 @@ class UnknownAttribute(ClientSideError):
         else:
             self.fieldname = name
         super(UnknownAttribute, self).__init__(self.msg)
+
+
+class AgentInProgress(IronicException):
+    _msg_fmt = _('Node %(node)s command "%(command)s" failed. Agent is '
+                 'presently executing a command. Error %(error)s')
+
+
+class InsufficentMemory(IronicException):
+    _msg_fmt = _("Available memory at %(free)s, Insufficent as %(required)s "
+                 "is required to proceed at this time.")
