@@ -81,7 +81,7 @@ def _get_raid_adapter(node):
     :returns: RAID adapter dictionary, None otherwise.
     :raises: IRMCOperationError on an error from python-scciclient.
     """
-    irmc_info = node.driver_info
+    irmc_info = irmc_common.parse_driver_info(node)
     LOG.info('iRMC driver is gathering RAID adapter info for node %s',
              node.uuid)
     try:
@@ -137,7 +137,7 @@ def _create_raid_adapter(node):
     :raises: IRMCOperationError on an error from python-scciclient.
     """
 
-    irmc_info = node.driver_info
+    irmc_info = irmc_common.parse_driver_info(node)
     target_raid_config = node.target_raid_config
 
     try:
@@ -164,7 +164,7 @@ def _delete_raid_adapter(node):
     :raises: IRMCOperationError if SCCI failed from python-scciclient.
     """
 
-    irmc_info = node.driver_info
+    irmc_info = irmc_common.parse_driver_info(node)
 
     try:
         client.elcm.delete_raid_configuration(irmc_info)
@@ -434,6 +434,13 @@ class IRMCRAID(base.RAIDInterface):
         node_list = manager.iter_nodes(fields=fields, filters=filters)
         for (node_uuid, driver, conductor_group, raid_config) in node_list:
             try:
+                # NOTE(TheJulia): Evaluate based upon presence of raid
+                # configuration before triggering a task, as opposed to after
+                # so we don't create excess node task objects with related
+                # DB queries.
+                if not raid_config or raid_config.get('fgi_status'):
+                    continue
+
                 lock_purpose = 'checking async RAID configuration tasks'
                 with task_manager.acquire(context, node_uuid,
                                           purpose=lock_purpose,
@@ -444,8 +451,6 @@ class IRMCRAID(base.RAIDInterface):
                         continue
                     if task.node.target_raid_config is None:
                         continue
-                    if not raid_config or raid_config.get('fgi_status'):
-                        continue
                     task.upgrade_lock()
                     if node.provision_state != states.CLEANWAIT:
                         continue
