@@ -168,13 +168,17 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         'network_data': object_fields.FlexibleDictField(nullable=True),
     }
 
-    def as_dict(self, secure=False):
+    def as_dict(self, secure=False, mask_configdrive=True):
         d = super(Node, self).as_dict()
         if secure:
             d['driver_info'] = strutils.mask_dict_password(
                 d.get('driver_info', {}), "******")
-            d['instance_info'] = strutils.mask_dict_password(
-                d.get('instance_info', {}), "******")
+            iinfo = d.pop('instance_info', {})
+            if not mask_configdrive:
+                configdrive = iinfo.pop('configdrive', None)
+            d['instance_info'] = strutils.mask_dict_password(iinfo, "******")
+            if not mask_configdrive and configdrive:
+                d['instance_info']['configdrive'] = configdrive
             d['driver_internal_info'] = strutils.mask_dict_password(
                 d.get('driver_internal_info', {}), "******")
         return d
@@ -208,12 +212,14 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
             raise exception.InvalidParameterValue(msg)
 
     def _set_from_db_object(self, context, db_object, fields=None):
-        fields = set(fields or self.fields) - {'traits'}
-        super(Node, self)._set_from_db_object(context, db_object, fields)
-        self.traits = object_base.obj_make_list(
-            context, objects.TraitList(context),
-            objects.Trait, db_object['traits'])
-        self.traits.obj_reset_changes()
+        use_fields = set(fields or self.fields) - {'traits'}
+        super(Node, self)._set_from_db_object(context, db_object, use_fields)
+        if not fields or 'traits' in fields:
+            self.traits = object_base.obj_make_list(
+                context, objects.TraitList(context),
+                objects.Trait, db_object['traits'],
+                fields=['trait', 'version'])
+            self.traits.obj_reset_changes()
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
     # methods can be used in the future to replace current explicit RPC calls.
@@ -308,7 +314,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     # @object_base.remotable_classmethod
     @classmethod
     def list(cls, context, limit=None, marker=None, sort_key=None,
-             sort_dir=None, filters=None):
+             sort_dir=None, filters=None, fields=None):
         """Return a list of Node objects.
 
         :param cls: the :class:`Node`
@@ -318,13 +324,29 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         :param sort_key: column to sort results by.
         :param sort_dir: direction to sort. "asc" or "desc".
         :param filters: Filters to apply.
+        :param fields: Requested fields to be returned. Please note, some
+                       fields are mandatory for the data model and are
+                       automatically included. These are: id, version,
+                       updated_at, created_at, owner, and lessee.
         :returns: a list of :class:`Node` object.
-
         """
+        if fields:
+            # All requests must include version, updated_at, created_at
+            # owner, and lessee to support access controls and database
+            # version model updates. Driver and conductor_group are required
+            # for conductor mapping.
+            target_fields = ['id'] + fields[:] + ['version', 'updated_at',
+                                                  'created_at', 'owner',
+                                                  'lessee', 'driver',
+                                                  'conductor_group']
+        else:
+            target_fields = None
+
         db_nodes = cls.dbapi.get_node_list(filters=filters, limit=limit,
                                            marker=marker, sort_key=sort_key,
-                                           sort_dir=sort_dir)
-        return cls._from_db_object_list(context, db_nodes)
+                                           sort_dir=sort_dir,
+                                           fields=target_fields)
+        return cls._from_db_object_list(context, db_nodes, target_fields)
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
     # methods can be used in the future to replace current explicit RPC calls.
