@@ -63,6 +63,7 @@ PARSED_IFNO = {
     'irmc_snmp_version': 'v2c',
     'irmc_snmp_security': None,
     'irmc_sensor_method': 'ipmitool',
+    'irmc_verify_ca': True,
 }
 
 
@@ -74,13 +75,14 @@ class IRMCDeployPrivateMethodsTestCase(test_common.BaseIRMCTest):
     def setUp(self):
         super(IRMCDeployPrivateMethodsTestCase, self).setUp()
 
-        CONF.irmc.remote_image_share_root = '/remote_image_share_root'
-        CONF.irmc.remote_image_server = '10.20.30.40'
-        CONF.irmc.remote_image_share_type = 'NFS'
-        CONF.irmc.remote_image_share_name = 'share'
-        CONF.irmc.remote_image_user_name = 'admin'
-        CONF.irmc.remote_image_user_password = 'admin0'
-        CONF.irmc.remote_image_user_domain = 'local'
+        CONF.set_override('remote_image_share_root',
+                          '/remote_image_share_root', 'irmc')
+        CONF.set_override('remote_image_server', '10.20.30.40', 'irmc')
+        CONF.set_override('remote_image_share_type', 'NFS', 'irmc')
+        CONF.set_override('remote_image_share_name', 'share', 'irmc')
+        CONF.set_override('remote_image_user_name', 'admin', 'irmc')
+        CONF.set_override('remote_image_user_password', 'admin0', 'irmc')
+        CONF.set_override('remote_image_user_domain', 'local', 'irmc')
 
     @mock.patch.object(os.path, 'isdir', spec_set=True, autospec=True)
     def test__parse_config_option(self, isdir_mock,
@@ -95,7 +97,8 @@ class IRMCDeployPrivateMethodsTestCase(test_common.BaseIRMCTest):
     @mock.patch.object(os.path, 'isdir', spec_set=True, autospec=True)
     def test__parse_config_option_non_existed_root(
             self, isdir_mock, check_share_fs_mounted_mock):
-        CONF.irmc.remote_image_share_root = '/non_existed_root'
+        CONF.set_override('remote_image_share_root', '/non_existed_root',
+                          'irmc')
         isdir_mock.return_value = False
 
         self.assertRaises(exception.InvalidParameterValue,
@@ -168,7 +171,7 @@ class IRMCDeployPrivateMethodsTestCase(test_common.BaseIRMCTest):
     def test__parse_instance_info_with_boot_iso_file_name_ok(
             self, check_share_fs_mounted_mock):
         """With optional 'irmc_boot_iso' file name."""
-        CONF.irmc.remote_image_share_root = '/etc'
+        CONF.set_override('remote_image_share_root', '/etc', 'irmc')
         self.node.instance_info['irmc_boot_iso'] = 'hosts'
         instance_info_expected = {'irmc_boot_iso': 'hosts'}
         instance_info_actual = irmc_boot._parse_instance_info(self.node)
@@ -178,7 +181,7 @@ class IRMCDeployPrivateMethodsTestCase(test_common.BaseIRMCTest):
     def test__parse_instance_info_without_boot_iso_ok(
             self, check_share_fs_mounted_mock):
         """With optional no 'irmc_boot_iso' file name."""
-        CONF.irmc.remote_image_share_root = '/etc'
+        CONF.set_override('remote_image_share_root', '/etc', 'irmc')
 
         self.node.instance_info['irmc_boot_iso'] = None
         instance_info_expected = {}
@@ -242,7 +245,7 @@ class IRMCDeployPrivateMethodsTestCase(test_common.BaseIRMCTest):
     @mock.patch.object(os.path, 'isfile', spec_set=True, autospec=True)
     def test__parse_instance_info_with_boot_iso_invalid(
             self, isfile_mock, check_share_fs_mounted_mock):
-        CONF.irmc.remote_image_share_root = '/etc'
+        CONF.set_override('remote_image_share_root', '/etc', 'irmc')
         isfile_mock.return_value = False
 
         with task_manager.acquire(self.context, self.node.uuid) as task:
@@ -264,7 +267,7 @@ class IRMCDeployPrivateMethodsTestCase(test_common.BaseIRMCTest):
     def test_parse_deploy_info_ok(self, mock_isfile,
                                   get_image_instance_info_mock,
                                   check_share_fs_mounted_mock):
-        CONF.irmc.remote_image_share_root = '/etc'
+        CONF.set_override('remote_image_share_root', '/etc', 'irmc')
         get_image_instance_info_mock.return_value = {'a': 'b'}
         driver_info_expected = {'a': 'b',
                                 'irmc_deploy_iso': 'hosts',
@@ -347,7 +350,7 @@ class IRMCDeployPrivateMethodsTestCase(test_common.BaseIRMCTest):
             setup_vmedia_mock,
             set_boot_device_mock,
             check_share_fs_mounted_mock):
-        CONF.irmc.remote_image_share_root = '/'
+        CONF.set_override('remote_image_share_root', '/', 'irmc')
 
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
@@ -380,7 +383,7 @@ class IRMCDeployPrivateMethodsTestCase(test_common.BaseIRMCTest):
             setup_vmedia_mock,
             set_boot_device_mock,
             check_share_fs_mounted_mock):
-        CONF.irmc.remote_image_share_root = '/'
+        CONF.set_override('remote_image_share_root', '/', 'irmc')
 
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
@@ -454,7 +457,7 @@ class IRMCDeployPrivateMethodsTestCase(test_common.BaseIRMCTest):
                                         boot_mode_mock,
                                         create_boot_iso_mock,
                                         check_share_fs_mounted_mock):
-        CONF.irmc.remote_image_share_root = '/'
+        CONF.set_override('remote_image_share_root', '/', 'irmc')
         image = '733d1c44-a2ea-414b-aca7-69decf20d810'
         is_image_href_ordinary_file_name_mock.return_value = False
         deploy_info_mock.return_value = {'irmc_boot_iso': image}
@@ -715,7 +718,7 @@ class IRMCDeployPrivateMethodsTestCase(test_common.BaseIRMCTest):
                        autospec=True)
     def test__remove_share_file(self, unlink_without_raise_mock,
                                 check_share_fs_mounted_mock):
-        CONF.irmc.remote_image_share_root = '/share'
+        CONF.set_override('remote_image_share_root', '/share', 'irmc')
 
         irmc_boot._remove_share_file("boot.iso")
 
@@ -731,12 +734,12 @@ class IRMCDeployPrivateMethodsTestCase(test_common.BaseIRMCTest):
         cd_set_params = (irmc_boot.scci
                          .get_virtual_cd_set_params_cmd.return_value)
 
-        CONF.irmc.remote_image_server = '10.20.30.40'
-        CONF.irmc.remote_image_user_domain = 'local'
-        CONF.irmc.remote_image_share_type = 'NFS'
-        CONF.irmc.remote_image_share_name = 'share'
-        CONF.irmc.remote_image_user_name = 'admin'
-        CONF.irmc.remote_image_user_password = 'admin0'
+        CONF.set_override('remote_image_server', '10.20.30.40', 'irmc')
+        CONF.set_override('remote_image_share_type', 'NFS', 'irmc')
+        CONF.set_override('remote_image_share_name', 'share', 'irmc')
+        CONF.set_override('remote_image_user_name', 'admin', 'irmc')
+        CONF.set_override('remote_image_user_password', 'admin0', 'irmc')
+        CONF.set_override('remote_image_user_domain', 'local', 'irmc')
 
         irmc_boot.scci.get_share_type.return_value = 0
 
@@ -813,12 +816,12 @@ class IRMCDeployPrivateMethodsTestCase(test_common.BaseIRMCTest):
         fd_set_params = (irmc_boot.scci
                          .get_virtual_fd_set_params_cmd.return_value)
 
-        CONF.irmc.remote_image_server = '10.20.30.40'
-        CONF.irmc.remote_image_user_domain = 'local'
-        CONF.irmc.remote_image_share_type = 'NFS'
-        CONF.irmc.remote_image_share_name = 'share'
-        CONF.irmc.remote_image_user_name = 'admin'
-        CONF.irmc.remote_image_user_password = 'admin0'
+        CONF.set_override('remote_image_server', '10.20.30.40', 'irmc')
+        CONF.set_override('remote_image_share_type', 'NFS', 'irmc')
+        CONF.set_override('remote_image_share_name', 'share', 'irmc')
+        CONF.set_override('remote_image_user_name', 'admin', 'irmc')
+        CONF.set_override('remote_image_user_password', 'admin0', 'irmc')
+        CONF.set_override('remote_image_user_domain', 'local', 'irmc')
 
         irmc_boot.scci.get_share_type.return_value = 0
 
@@ -896,8 +899,8 @@ class IRMCDeployPrivateMethodsTestCase(test_common.BaseIRMCTest):
         # irmc_boot.check_share_fs_mounted is mocked in
         # third_party_driver_mocks.py.
         # irmc_boot.check_share_fs_mounted_orig is the real function.
-        CONF.irmc.remote_image_share_root = '/'
-        CONF.irmc.remote_image_share_type = 'nfs'
+        CONF.set_override('remote_image_share_root', '/', 'irmc')
+        CONF.set_override('remote_image_share_type', 'nfs', 'irmc')
         result = irmc_boot.check_share_fs_mounted_orig()
 
         parse_conf_mock.assert_called_once_with()
@@ -913,8 +916,8 @@ class IRMCDeployPrivateMethodsTestCase(test_common.BaseIRMCTest):
         # irmc_boot.check_share_fs_mounted is mocked in
         # third_party_driver_mocks.py.
         # irmc_boot.check_share_fs_mounted_orig is the real function.
-        CONF.irmc.remote_image_share_root = '/etc'
-        CONF.irmc.remote_image_share_type = 'cifs'
+        CONF.set_override('remote_image_share_root', '/etc', 'irmc')
+        CONF.set_override('remote_image_share_type', 'cifs', 'irmc')
 
         self.assertRaises(exception.IRMCSharedFileSystemNotMounted,
                           irmc_boot.check_share_fs_mounted_orig)
@@ -1384,121 +1387,27 @@ class IRMCPXEBootTestCase(test_common.BaseIRMCTest):
             mock_parent_prepare.assert_called_once_with(
                 task.driver.boot, task, {})
 
-    @mock.patch.object(irmc_common, 'set_secure_boot_mode', spec_set=True,
-                       autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'prepare_instance', spec_set=True,
                        autospec=True)
-    def test_prepare_instance_with_secure_boot(self, mock_prepare_instance,
-                                               mock_set_secure_boot_mode):
-        self.node.provision_state = states.DEPLOYING
-        self.node.target_provision_state = states.ACTIVE
-        self.node.instance_info = {
-            'capabilities': {
-                "secure_boot": "true"
-            }
-        }
-        self.node.save()
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=False) as task:
-            task.driver.boot.prepare_instance(task)
-            mock_set_secure_boot_mode.assert_called_once_with(task.node,
-                                                              enable=True)
-            mock_prepare_instance.assert_called_once_with(
-                task.driver.boot, task)
-
-    @mock.patch.object(irmc_common, 'set_secure_boot_mode', spec_set=True,
-                       autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'prepare_instance', spec_set=True,
-                       autospec=True)
-    def test_prepare_instance_with_secure_boot_false(
-            self, mock_prepare_instance, mock_set_secure_boot_mode):
-        self.node.provision_state = states.DEPLOYING
-        self.node.target_provision_state = states.ACTIVE
-        self.node.instance_info = {
-            'capabilities': {
-                "secure_boot": "false"
-            }
-        }
-        self.node.save()
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=False) as task:
-            task.driver.boot.prepare_instance(task)
-            self.assertFalse(mock_set_secure_boot_mode.called)
-            mock_prepare_instance.assert_called_once_with(
-                task.driver.boot, task)
-
-    @mock.patch.object(irmc_common, 'set_secure_boot_mode', spec_set=True,
-                       autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'prepare_instance', spec_set=True,
-                       autospec=True)
-    def test_prepare_instance_without_secure_boot(self, mock_prepare_instance,
-                                                  mock_set_secure_boot_mode):
+    def test_prepare_instance(self, mock_prepare_instance):
         self.node.provision_state = states.DEPLOYING
         self.node.target_provision_state = states.ACTIVE
         self.node.save()
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             task.driver.boot.prepare_instance(task)
-            self.assertFalse(mock_set_secure_boot_mode.called)
             mock_prepare_instance.assert_called_once_with(
                 task.driver.boot, task)
 
-    @mock.patch.object(irmc_common, 'set_secure_boot_mode', spec_set=True,
-                       autospec=True)
     @mock.patch.object(pxe.PXEBoot, 'clean_up_instance', spec_set=True,
                        autospec=True)
-    def test_clean_up_instance_with_secure_boot(self, mock_clean_up_instance,
-                                                mock_set_secure_boot_mode):
-        self.node.provision_state = states.CLEANING
-        self.node.target_provision_state = states.AVAILABLE
-        self.node.instance_info = {
-            'capabilities': {
-                "secure_boot": "true"
-            }
-        }
-        self.node.save()
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=False) as task:
-            task.driver.boot.clean_up_instance(task)
-            mock_set_secure_boot_mode.assert_called_once_with(task.node,
-                                                              enable=False)
-            mock_clean_up_instance.assert_called_once_with(
-                task.driver.boot, task)
-
-    @mock.patch.object(irmc_common, 'set_secure_boot_mode', spec_set=True,
-                       autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'clean_up_instance', spec_set=True,
-                       autospec=True)
-    def test_clean_up_instance_secure_boot_false(self, mock_clean_up_instance,
-                                                 mock_set_secure_boot_mode):
-        self.node.provision_state = states.CLEANING
-        self.node.target_provision_state = states.AVAILABLE
-        self.node.instance_info = {
-            'capabilities': {
-                "secure_boot": "false"
-            }
-        }
-        self.node.save()
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=False) as task:
-            task.driver.boot.clean_up_instance(task)
-            self.assertFalse(mock_set_secure_boot_mode.called)
-            mock_clean_up_instance.assert_called_once_with(
-                task.driver.boot, task)
-
-    @mock.patch.object(irmc_common, 'set_secure_boot_mode', spec_set=True,
-                       autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'clean_up_instance', spec_set=True,
-                       autospec=True)
-    def test_clean_up_instance_without_secure_boot(
-            self, mock_clean_up_instance, mock_set_secure_boot_mode):
+    def test_clean_up_instance(self, mock_clean_up_instance):
         self.node.provision_state = states.CLEANING
         self.node.target_provision_state = states.AVAILABLE
         self.node.save()
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             task.driver.boot.clean_up_instance(task)
-            self.assertFalse(mock_set_secure_boot_mode.called)
             mock_clean_up_instance.assert_called_once_with(
                 task.driver.boot, task)
 
