@@ -19,7 +19,6 @@ import os
 from unittest import mock
 
 from oslo_config import cfg
-from oslo_serialization import jsonutils as json
 from oslo_utils import uuidutils
 
 from ironic.common import boot_devices
@@ -34,6 +33,7 @@ from ironic.conductor import task_manager
 from ironic.conductor import utils as manager_utils
 from ironic.drivers import base as drivers_base
 from ironic.drivers.modules import agent_base
+from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import ipxe
 from ironic.drivers.modules import pxe_base
@@ -143,7 +143,9 @@ class iPXEBootTestCase(db_base.DbTestCase):
             self.assertTrue(mock_boot_option.called)
             self.assertTrue(mock_glance.called)
 
-    def test_validate_with_boot_iso_and_image_source(self):
+    @mock.patch('ironic.drivers.modules.deploy_utils.get_boot_option',
+                return_value='ramdisk', autospec=True)
+    def test_validate_with_boot_iso_and_image_source(self, mock_boot_option):
         i_info = self.node.instance_info
         i_info['image_source'] = "http://localhost:1234/image"
         i_info['boot_iso'] = "http://localhost:1234/boot.iso"
@@ -154,16 +156,26 @@ class iPXEBootTestCase(db_base.DbTestCase):
             self.assertRaises(exception.InvalidParameterValue,
                               task.driver.boot.validate,
                               task)
+            mock_boot_option.assert_called_once_with(task.node)
 
-    def test_validate_fail_missing_image_source(self):
-        info = dict(INST_INFO_DICT)
-        del info['image_source']
-        self.node.instance_info = json.dumps(info)
+    @mock.patch('ironic.drivers.modules.deploy_utils.get_boot_option',
+                return_value='local', autospec=True)
+    def test_validate_no_image_source_for_local_boot(self, mock_boot_option):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
-            task.node['instance_info'] = json.dumps(info)
+            del task.node['instance_info']['image_source']
+            task.driver.boot.validate(task)
+            mock_boot_option.assert_called_with(task.node)
+
+    @mock.patch('ironic.drivers.modules.deploy_utils.get_boot_option',
+                return_value='netboot', autospec=True)
+    def test_validate_fail_missing_image_source(self, mock_boot_option):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            del task.node['instance_info']['image_source']
             self.assertRaises(exception.MissingParameterValue,
                               task.driver.boot.validate, task)
+            mock_boot_option.assert_called_with(task.node)
 
     def test_validate_fail_no_port(self):
         new_node = obj_utils.create_test_node(
@@ -211,18 +223,25 @@ class iPXEBootTestCase(db_base.DbTestCase):
                               task.driver.boot.validate,
                               task)
 
+    @mock.patch('ironic.drivers.modules.deploy_utils.get_boot_option',
+                return_value='netboot', autospec=True)
     @mock.patch.object(image_service.GlanceImageService, 'show',
                        autospec=True)
-    def test_validate_fail_glance_image_doesnt_exists(self, mock_glance):
+    def test_validate_fail_glance_image_doesnt_exists(self, mock_glance,
+                                                      mock_boot_option):
         mock_glance.side_effect = exception.ImageNotFound('not found')
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             self.assertRaises(exception.InvalidParameterValue,
                               task.driver.boot.validate, task)
+            mock_boot_option.assert_called_with(task.node)
 
+    @mock.patch('ironic.drivers.modules.deploy_utils.get_boot_option',
+                return_value='netboot', autospec=True)
     @mock.patch.object(image_service.GlanceImageService, 'show',
                        autospec=True)
-    def test_validate_fail_glance_conn_problem(self, mock_glance):
+    def test_validate_fail_glance_conn_problem(self, mock_glance,
+                                               mock_boot_option):
         exceptions = (exception.GlanceConnectionFailed('connection fail'),
                       exception.ImageNotAuthorized('not authorized'),
                       exception.Invalid('invalid'))
@@ -232,6 +251,7 @@ class iPXEBootTestCase(db_base.DbTestCase):
                                       shared=True) as task:
                 self.assertRaises(exception.InvalidParameterValue,
                                   task.driver.boot.validate, task)
+                mock_boot_option.assert_called_with(task.node)
 
     def test_validate_inspection(self):
         with task_manager.acquire(self.context, self.node.uuid) as task:
@@ -342,80 +362,6 @@ class iPXEBootTestCase(db_base.DbTestCase):
         self.node.save()
         self._test_prepare_ramdisk()
 
-    def test_prepare_ramdisk_force_persistent_boot_device_true(self):
-        self.node.provision_state = states.DEPLOYING
-        driver_info = self.node.driver_info
-        driver_info['force_persistent_boot_device'] = 'True'
-        self.node.driver_info = driver_info
-        self.node.save()
-        self._test_prepare_ramdisk(persistent=True)
-
-    def test_prepare_ramdisk_force_persistent_boot_device_bool_true(self):
-        self.node.provision_state = states.DEPLOYING
-        driver_info = self.node.driver_info
-        driver_info['force_persistent_boot_device'] = True
-        self.node.driver_info = driver_info
-        self.node.save()
-        self._test_prepare_ramdisk(persistent=True)
-
-    def test_prepare_ramdisk_force_persistent_boot_device_sloppy_true(self):
-        for value in ['true', 't', '1', 'on', 'y', 'YES']:
-            self.node.provision_state = states.DEPLOYING
-            driver_info = self.node.driver_info
-            driver_info['force_persistent_boot_device'] = value
-            self.node.driver_info = driver_info
-            self.node.save()
-            self._test_prepare_ramdisk(persistent=True)
-
-    def test_prepare_ramdisk_force_persistent_boot_device_false(self):
-        self.node.provision_state = states.DEPLOYING
-        driver_info = self.node.driver_info
-        driver_info['force_persistent_boot_device'] = 'False'
-        self.node.driver_info = driver_info
-        self.node.save()
-        self._test_prepare_ramdisk()
-
-    def test_prepare_ramdisk_force_persistent_boot_device_bool_false(self):
-        self.node.provision_state = states.DEPLOYING
-        driver_info = self.node.driver_info
-        driver_info['force_persistent_boot_device'] = False
-        self.node.driver_info = driver_info
-        self.node.save()
-        self._test_prepare_ramdisk(persistent=False)
-
-    def test_prepare_ramdisk_force_persistent_boot_device_sloppy_false(self):
-        for value in ['false', 'f', '0', 'off', 'n', 'NO', 'yxz']:
-            self.node.provision_state = states.DEPLOYING
-            driver_info = self.node.driver_info
-            driver_info['force_persistent_boot_device'] = value
-            self.node.driver_info = driver_info
-            self.node.save()
-            self._test_prepare_ramdisk()
-
-    def test_prepare_ramdisk_force_persistent_boot_device_default(self):
-        self.node.provision_state = states.DEPLOYING
-        driver_info = self.node.driver_info
-        driver_info['force_persistent_boot_device'] = 'Default'
-        self.node.driver_info = driver_info
-        self.node.save()
-        self._test_prepare_ramdisk(persistent=False)
-
-    def test_prepare_ramdisk_force_persistent_boot_device_always(self):
-        self.node.provision_state = states.DEPLOYING
-        driver_info = self.node.driver_info
-        driver_info['force_persistent_boot_device'] = 'Always'
-        self.node.driver_info = driver_info
-        self.node.save()
-        self._test_prepare_ramdisk(persistent=True)
-
-    def test_prepare_ramdisk_force_persistent_boot_device_never(self):
-        self.node.provision_state = states.DEPLOYING
-        driver_info = self.node.driver_info
-        driver_info['force_persistent_boot_device'] = 'Never'
-        self.node.driver_info = driver_info
-        self.node.save()
-        self._test_prepare_ramdisk(persistent=False)
-
     def test_prepare_ramdisk_rescue(self):
         self.node.provision_state = states.RESCUING
         self.node.save()
@@ -673,7 +619,8 @@ class iPXEBootTestCase(db_base.DbTestCase):
             provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
             switch_pxe_config_mock.assert_called_once_with(
                 pxe_config_path, "30212642-09d3-467f-8e09-21685826ab50",
-                'bios', False, False, False, False, ipxe_enabled=True)
+                'bios', False, False, False, False, ipxe_enabled=True,
+                anaconda_boot=False)
             set_boot_device_mock.assert_called_once_with(task,
                                                          boot_devices.PXE,
                                                          persistent=True)
@@ -722,7 +669,8 @@ class iPXEBootTestCase(db_base.DbTestCase):
                 ipxe_enabled=True)
             switch_pxe_config_mock.assert_called_once_with(
                 pxe_config_path, "30212642-09d3-467f-8e09-21685826ab50",
-                'bios', False, False, False, False, ipxe_enabled=True)
+                'bios', False, False, False, False, ipxe_enabled=True,
+                anaconda_boot=False)
             self.assertFalse(set_boot_device_mock.called)
 
     @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
@@ -839,7 +787,8 @@ class iPXEBootTestCase(db_base.DbTestCase):
                 ipxe_enabled=True)
             switch_pxe_config_mock.assert_called_once_with(
                 pxe_config_path, None, boot_modes.LEGACY_BIOS, False,
-                ipxe_enabled=True, iscsi_boot=True, ramdisk_boot=False)
+                ipxe_enabled=True, iscsi_boot=True, ramdisk_boot=False,
+                anaconda_boot=False)
             set_boot_device_mock.assert_called_once_with(task,
                                                          boot_devices.PXE,
                                                          persistent=True)
@@ -885,15 +834,77 @@ class iPXEBootTestCase(db_base.DbTestCase):
                 ipxe_enabled=True)
             switch_pxe_config_mock.assert_called_once_with(
                 pxe_config_path, None, boot_modes.LEGACY_BIOS, False,
-                ipxe_enabled=True, iscsi_boot=False, ramdisk_boot=True)
+                ipxe_enabled=True, iscsi_boot=False, ramdisk_boot=True,
+                anaconda_boot=False)
             set_boot_device_mock.assert_called_once_with(task,
                                                          boot_devices.PXE,
                                                          persistent=True)
 
+    @mock.patch('os.path.isfile', lambda filename: False)
+    @mock.patch.object(pxe_utils, 'create_pxe_config', autospec=True)
+    @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
+    @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
+    @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
+    @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
+    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
+    def test_prepare_instance_netboot_ramdisk_with_kernel_arg(
+            self, get_image_info_mock, cache_mock,
+            dhcp_factory_mock, switch_pxe_config_mock,
+            set_boot_device_mock, create_pxe_config_mock):
+        http_url = 'http://192.1.2.3:1234'
+        self.config(http_url=http_url, group='deploy')
+        self.config(enabled_deploy_interfaces='ramdisk')
+        provider_mock = mock.MagicMock()
+        dhcp_factory_mock.return_value = provider_mock
+        self.node.instance_info = {'ramdisk_kernel_arguments': 'cat meow'}
+        image_info = {'kernel': ('', '/path/to/kernel'),
+                      'deploy_kernel': ('', '/path/to/kernel'),
+                      'ramdisk': ('', '/path/to/ramdisk'),
+                      'deploy_ramdisk': ('', '/path/to/ramdisk')}
+        get_image_info_mock.return_value = image_info
+        self.node.provision_state = states.DEPLOYING
+        self.node.deploy_interface = 'ramdisk'
+        self.node.save()
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            dhcp_opts = pxe_utils.dhcp_options_for_instance(task,
+                                                            ipxe_enabled=True)
+            dhcp_opts += pxe_utils.dhcp_options_for_instance(
+                task, ipxe_enabled=True, ip_version=6)
+            pxe_config_path = pxe_utils.get_pxe_config_file_path(
+                task.node.uuid, ipxe_enabled=True)
+            task.driver.boot.prepare_instance(task)
+            self.assertTrue(get_image_info_mock.called)
+            self.assertTrue(cache_mock.called)
+            uuid = self.node.uuid
+            expected_params = {
+                'aki_path': 'http://192.1.2.3:1234/' + uuid + '/kernel',
+                'ari_path': 'http://192.1.2.3:1234/' + uuid + '/ramdisk',
+                'ramdisk_opts': 'cat meow',
+                'pxe_append_params': 'nofb nomodeset vga=normal ipa-debug=1 '
+                                     'ipa-global-request-'
+                                     'id=' + task.context.request_id,
+                'tftp_server': mock.ANY,
+                'ipxe_timeout': 0
+            }
+            provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
+            create_pxe_config_mock.assert_called_once_with(
+                task, expected_params, CONF.pxe.ipxe_config_template,
+                ipxe_enabled=True)
+            switch_pxe_config_mock.assert_called_once_with(
+                pxe_config_path, None, boot_modes.LEGACY_BIOS, False,
+                ipxe_enabled=True, iscsi_boot=False, ramdisk_boot=True,
+                anaconda_boot=False)
+            set_boot_device_mock.assert_called_once_with(task,
+                                                         boot_devices.PXE,
+                                                         persistent=True)
+
+    @mock.patch.object(boot_mode_utils, 'configure_secure_boot_if_needed',
+                       autospec=True)
     @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
     @mock.patch.object(pxe_utils, 'clean_up_pxe_config', autospec=True)
     def test_prepare_instance_localboot(self, clean_up_pxe_config_mock,
-                                        set_boot_device_mock):
+                                        set_boot_device_mock,
+                                        secure_boot_mock):
         with task_manager.acquire(self.context, self.node.uuid) as task:
             instance_info = task.node.instance_info
             instance_info['capabilities'] = {'boot_option': 'local'}
@@ -905,6 +916,7 @@ class iPXEBootTestCase(db_base.DbTestCase):
             set_boot_device_mock.assert_called_once_with(task,
                                                          boot_devices.DISK,
                                                          persistent=True)
+            secure_boot_mock.assert_called_once_with(task)
 
     @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
     @mock.patch.object(pxe_utils, 'clean_up_pxe_config', autospec=True)
@@ -949,7 +961,8 @@ class iPXEBootTestCase(db_base.DbTestCase):
                                                          persistent=True)
             switch_pxe_config_mock.assert_called_once_with(
                 pxe_config_path, "30212642-09d3-467f-8e09-21685826ab50",
-                'bios', True, False, False, False, ipxe_enabled=True)
+                'bios', True, False, False, False, ipxe_enabled=True,
+                anaconda_boot=False)
             # No clean up
             self.assertFalse(clean_up_pxe_config_mock.called)
             # No netboot configuration beyond the PXE files
@@ -957,10 +970,13 @@ class iPXEBootTestCase(db_base.DbTestCase):
             self.assertFalse(cache_mock.called)
             self.assertFalse(dhcp_factory_mock.return_value.update_dhcp.called)
 
+    @mock.patch.object(boot_mode_utils, 'deconfigure_secure_boot_if_needed',
+                       autospec=True)
     @mock.patch.object(pxe_utils, 'clean_up_pxe_env', autospec=True)
     @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
     def test_clean_up_instance(self, get_image_info_mock,
-                               clean_up_pxe_env_mock):
+                               clean_up_pxe_env_mock,
+                               secure_boot_mock):
         with task_manager.acquire(self.context, self.node.uuid) as task:
             image_info = {'kernel': ['', '/path/to/kernel'],
                           'ramdisk': ['', '/path/to/ramdisk']}
@@ -970,6 +986,7 @@ class iPXEBootTestCase(db_base.DbTestCase):
                 task, image_info, ipxe_enabled=True)
             get_image_info_mock.assert_called_once_with(
                 task, ipxe_enabled=True)
+            secure_boot_mock.assert_called_once_with(task)
 
 
 @mock.patch.object(ipxe.iPXEBoot, '__init__', lambda self: None)
